/*

  Copyright 2010, 2011 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package pado.api;

import cf.api.ApplicationKnowledge;
import cf.api.CyberApplication;

import org.opt4j.core.Individual;
import org.opt4j.core.Objectives;

/* Individual Knowledge */
public class BestIndividualKnowledge extends ApplicationKnowledge
{
  Individual individual;
  int id; 

  public Individual individual() { return individual; }
  public int id() { return id; }

  public BestIndividualKnowledge(CyberApplication creator, double timeToExpire)
  {
    super(creator, timeToExpire);
	this.id = -1;
  }

  public BestIndividualKnowledge(CyberApplication creator, double timeToExpire, int id)
  {
    super(creator, timeToExpire);
	this.id = id; 
  }

  public BestIndividualKnowledge(CyberApplication creator, double timeToExpire, Individual individual, int id)
  {
    super(creator, timeToExpire);
	this.individual = individual;
	this.id = id; 
  }

  public boolean related(ApplicationKnowledge k)
  {
    return this.getClass() == k.getClass();
  }

  public boolean replaces(ApplicationKnowledge k)
  {
    BestIndividualKnowledge bik = (BestIndividualKnowledge)k;
    Objectives other = bik.individual.getObjectives();
    Objectives mine = individual.getObjectives();
    
	if(bik.id != id)	 
	  return false; 

	if(!mine.getKeys().equals(other.getKeys()))
	  return false; 
	
    if(mine.dominates(other))
	{
      return true;
    }
    else if(mine.isEqual(other) && creationTime() > bik.creationTime())
    {
      return true;
    }
    else if(mine.isEqual(other) && creationTime() == bik.creationTime() && creator().compareTo(bik.creator()) < 0) 
    {
      return true;
    }
    else
    {
      return false;
    }
  }

  public boolean subsumes(ApplicationKnowledge k)
  {
    BestIndividualKnowledge bik = (BestIndividualKnowledge)k;
    Objectives other = bik.individual.getObjectives();
    Objectives mine = individual.getObjectives();
    
	if(bik.id != id) 
	  return false; 
	
	if(!mine.getKeys().equals(other.getKeys()))
	  return false; 
	
    if(mine.isEqual(other) && creationTime() == bik.creationTime() && creator().equals(bik.creator())) 
    {
      return true;
    }
    else
    {
      return false;
    }
  }

  public String logRepLong()
  {
    if(individual.getObjectives().isFeasible())
      return creationTime() + " by " + creator() + " (id:" + id + ") "  + individual.getObjectives();
    else
      return "INFEASIBLE";
  }

  public String toString()
  {
    return logRepLong();
  }
}


