/*

  Copyright 2010, 2011 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package pado.app.tsp;

import pado.api.PADOParam;

import cf.common.Param;
import cf.real.RealEngine;
import cf.real.RealWorld;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.StringTokenizer;

class PADOLauncher extends RealEngine
{
  public RealWorld createRealWorld()
  {
    readParam();
    return new TSPWorld();
  }

  void readParam()
  {
    try
    {
      FileReader fr = new FileReader("param");
      if(fr == null)
        return;
      BufferedReader br = new BufferedReader( fr );
      String s = br.readLine();
      StringTokenizer st = new StringTokenizer(s);
      PADOParam.NUM_THREADS = Integer.parseInt(st.nextToken()); st = new StringTokenizer(br.readLine());
      PADOParam.NEIGHBOR_FACILITATOR =  st.nextToken();
    }
    catch(IOException e)
    {
      e.printStackTrace();
    }
  }
}

class TSPMain
{
    public static void main (String args[]) throws IOException
    {
    	if(!Param.RealMode) 
  		{
  			System.out.println("Param.RealMode should be true");
  			System.exit(0);
  		}
    	
    	if(Param.EnableMason) 
  		{
  			System.out.println("Param.EnableMason should be false");
  			System.exit(0);
  		}

    	PADOLauncher launch = new PADOLauncher();
    	launch.run();
    }
}
